@@ -7,4 +7,8 @@ pref("intl.locale.matchOS", true);
 // enable fixed-up typeaheadfind extension
 pref("accessibility.typeaheadfindsea", true);
 pref("accessibility.typeaheadfindsea.autostart", true);
+pref("accessibility.typeaheadfindsea.linksonly", true);
+
+// disable image resizing
+pref("browser.enable_automatic_image_resizing", false);
 
