@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ContentHandler.cpp,v 1.77 2005/02/26 22:01:48 chpe Exp $
+ *  $Id: ContentHandler.cpp,v 1.77.2.2 2005/03/24 15:48:51 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -93,15 +93,26 @@ GContentHandler::~GContentHandler()
 
 NS_IMPL_ISUPPORTS1(GContentHandler, nsIHelperAppLauncherDialog)
 
+#ifdef MOZ_NSIHELPERAPPLAUNCHERDIALOG_UNSIGNED
+/* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext, in unsigned long aReason); */
+NS_IMETHODIMP
+GContentHandler::Show (nsIHelperAppLauncher *aLauncher,
+		       nsISupports *aContext,
+		       PRUint32 aReason)
+#else
 /* void show (in nsIHelperAppLauncher aLauncher, in nsISupports aContext); */
-NS_IMETHODIMP GContentHandler::Show(nsIHelperAppLauncher *aLauncher,
-				    nsISupports *aContext,
-				    PRBool aForced)
+NS_IMETHODIMP
+GContentHandler::Show (nsIHelperAppLauncher *aLauncher,
+		       nsISupports *aContext,
+		       PRBool aForced)
+#endif
 {
 	nsresult rv;
 	EphyEmbedSingle *single;
 	gboolean handled = FALSE;
 
+	/* FIXME: handle aForced / aReason argument in some way? */
+
 	mContext = aContext;
 	mLauncher = aLauncher;
 	rv = Init ();
@@ -261,7 +272,7 @@ NS_METHOD GContentHandler::MIMEConfirmAction ()
 			(mAction == CONTENT_ACTION_OPEN_TMP) ?
 			GTK_STOCK_OPEN : EPHY_STOCK_DOWNLOAD;
 
-	if (mPermission == EPHY_MIME_PERMISSION_UNSAFE && mHelperApp)
+	if (mPermission != EPHY_MIME_PERMISSION_SAFE && mHelperApp)
 	{
 		dialog = gtk_message_dialog_new
 			(parentWindow, GTK_DIALOG_DESTROY_WITH_PARENT,
