@@ -15,21 +15,22 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyBrowser.cpp,v 1.60 2004/10/30 18:04:39 chpe Exp $
+ *  $Id: EphyBrowser.cpp,v 1.76 2005/01/28 22:23:38 chpe Exp $
  */
 
 #include "mozilla-config.h"
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #include "EphyBrowser.h"
 #include "EphyUtils.h"
+#include "EventContext.h"
 #include "ephy-embed.h"
 #include "ephy-string.h"
 #include "ephy-debug.h"
 #include "print-dialog.h"
+#include "mozilla-embed.h"
+#include "mozilla-embed-event.h"
 
 #include <gtkmozembed_internal.h>
 #include <unistd.h>
@@ -59,27 +60,49 @@
 #include "nsIDOMDocument.h"
 #include "nsIDOM3Document.h"
 #include "nsIDOMEvent.h"
+#include "nsIDOMKeyEvent.h"
+#include "nsIDOMMouseEvent.h"
 #include "nsIDOMNSEvent.h"
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMPopupBlockedEvent.h"
 #include "nsIDOMNode.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMWindow2.h"
+#include "nsIDOMDocumentView.h"
+#include "nsIDOMAbstractView.h"
+#define MOZILLA_STRICT_API
 #include "nsEmbedString.h"
+#undef MOZILLA_STRICT_API
 #include "nsMemory.h"
 #include "nsIChannel.h"
+#include "nsIScriptSecurityManager.h"
+#include "nsIServiceManager.h"
+#include "nsIInterfaceRequestor.h"
+#include "nsIDOMHTMLDocument.h"
+#include "nsIDOMXMLDocument.h"
 
 #ifdef ALLOW_PRIVATE_API
+#include "nsIImageDocument.h"
+/* not frozen yet */
+#include "nsIContentPolicy.h"
+/* will never be frozen */
 #include "nsIDocShell.h"
 #include "nsIMarkupDocumentViewer.h"
 #ifdef HAVE_MOZILLA_PSM
 /* not sure about this one: */
 #include <nsITransportSecurityInfo.h>
+/* these are in pipnss/, are they really private? */
+#include <nsISSLStatus.h>
+#include <nsISSLStatusProvider.h>
+#include <nsIX509Cert.h>
+#include <nsICertificateDialogs.h>
 #endif
 #endif
 
 static PRUnichar DOMLinkAdded[] = { 'D', 'O', 'M', 'L', 'i', 'n', 'k',
 				    'A', 'd', 'd', 'e', 'd', '\0' };
+static PRUnichar ContextMenu[] = { 'c', 'o', 'n', 't', 'e', 'x', 't', 'm',
+				   'e', 'n', 'u', '\0' };
 static PRUnichar DOMPopupBlocked[] = { 'D', 'O', 'M', 'P', 'o', 'p',
 				       'u', 'p', 'B', 'l', 'o', 'c',
 				       'k', 'e', 'd', '\0' };
@@ -91,29 +114,35 @@ static PRUnichar DOMModalDialogClosed[] = { 'D', 'O', 'M', 'M', 'o', 'd', 'a',
 					    'l', 'D', 'i', 'a', 'l', 'o', 'g',
 					    'C', 'l', 'o', 's', 'e', 'd', '\0' };
 
-EphyEventListener::EphyEventListener(void)
+EphyEventListener::EphyEventListener()
 : mOwner(nsnull)
 {
-	LOG ("EphyEventListener ctor (%p)", this)
+	LOG ("EphyEventListener ctor (%p)", this);
 }
 
 EphyEventListener::~EphyEventListener()
 {
-	LOG ("EphyEventListener dtor (%p)", this)
+	LOG ("EphyEventListener dtor (%p)", this);
 }
 
 NS_IMPL_ISUPPORTS1(EphyEventListener, nsIDOMEventListener)
 
 nsresult
-EphyEventListener::Init(EphyEmbed *aOwner)
+EphyEventListener::Init (EphyBrowser *aOwner)
 {
 	mOwner = aOwner;
 	return NS_OK;
 }
 
-nsresult
-EphyFaviconEventListener::HandleFaviconLink (nsIDOMNode *node)
+NS_IMETHODIMP
+EphyFaviconEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
 {
+	nsCOMPtr<nsIDOMEventTarget> eventTarget;
+	aDOMEvent->GetTarget(getter_AddRefs(eventTarget));
+
+	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(eventTarget);
+	NS_ENSURE_TRUE (node, NS_ERROR_FAILURE);
+
 	nsCOMPtr<nsIDOMElement> linkElement;
 	linkElement = do_QueryInterface (node);
 	if (!linkElement) return NS_ERROR_FAILURE;
@@ -142,48 +171,100 @@ EphyFaviconEventListener::HandleFaviconLink (nsIDOMNode *node)
 		node->GetOwnerDocument(getter_AddRefs(domDoc));
 		NS_ENSURE_TRUE (domDoc, NS_ERROR_FAILURE);
 
-		nsCOMPtr<nsIDOM3Document> doc = do_QueryInterface (domDoc);
+		nsCOMPtr<nsIDOMDocumentView> docView (do_QueryInterface (domDoc));
+		NS_ENSURE_TRUE (docView, NS_ERROR_FAILURE);
+
+		nsCOMPtr<nsIDOMAbstractView> abstractView;
+		docView->GetDefaultView (getter_AddRefs (abstractView));
+
+		nsCOMPtr<nsIDOMWindow> domWin (do_QueryInterface (abstractView));
+		NS_ENSURE_TRUE (domWin, NS_ERROR_FAILURE);
+
+		nsCOMPtr<nsIDOMWindow> topDomWin;
+		domWin->GetTop (getter_AddRefs (topDomWin));
+
+		nsCOMPtr<nsISupports> domWinAsISupports (do_QueryInterface (domWin));
+		nsCOMPtr<nsISupports> topDomWinAsISupports (do_QueryInterface (topDomWin));
+		/* disallow subframes to set favicon */
+		if (domWinAsISupports != topDomWinAsISupports) return NS_OK;
+
+		nsCOMPtr<nsIDOM3Document> doc (do_QueryInterface (domDoc));
 		NS_ENSURE_TRUE (doc, NS_ERROR_FAILURE);
 
 		nsEmbedString spec;
 		rv = doc->GetDocumentURI (spec);
-		NS_ENSURE_SUCCESS (rv, rv);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-		nsCOMPtr<nsIURI> uri;
-		rv = EphyUtils::NewURI (getter_AddRefs(uri), spec);
-		NS_ENSURE_SUCCESS (rv, rv);
+                nsEmbedCString encoding;
+                mOwner->GetEncoding (encoding);
 
-		nsEmbedCString favicon_url;
-		rv = uri->Resolve (link, favicon_url);
-		if (NS_FAILED (rv)) return NS_ERROR_FAILURE;
-		
-		char *url = g_strdup (favicon_url.get());
-		g_signal_emit_by_name (mOwner, "ge_favicon", url);
-		g_free (url);
+		nsCOMPtr<nsIURI> docUri;
+		EphyUtils::NewURI (getter_AddRefs(docUri), spec, encoding.get());
+		NS_ENSURE_TRUE (docUri, NS_ERROR_FAILURE);
+
+		nsEmbedCString faviconUrl;
+		rv = docUri->Resolve (link, faviconUrl);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+		nsCOMPtr<nsIURI> favUri;
+		EphyUtils::NewURI (getter_AddRefs (favUri), faviconUrl);
+		NS_ENSURE_TRUE (favUri, NS_ERROR_FAILURE);
+
+		/* check if load is allowed */
+		nsCOMPtr<nsIScriptSecurityManager> secMan
+			(do_GetService("@mozilla.org/scriptsecuritymanager;1"));
+		/* refuse if we can't check */
+		NS_ENSURE_TRUE (secMan, NS_OK);
+
+		rv = secMan->CheckLoadURI(docUri, favUri,
+					  nsIScriptSecurityManager::STANDARD);
+		/* failure means it didn't pass the security check */
+		if (NS_FAILED (rv)) return NS_OK;
+
+		/* security check passed, now check with content policy */
+		nsCOMPtr<nsIContentPolicy> policy =
+			do_GetService("@mozilla.org/layout/content-policy;1");
+		/* refuse if we can't check */
+		NS_ENSURE_TRUE (policy, NS_OK);
+
+#if MOZ_NSICONTENTPOLICY_VARIANT == 2
+		/* FIXME: mozilla tabbrowser.xml passes
+		 * safeGetProperty(event.target, "type") as mimetype guess:
+		 */
+		PRUnichar typeAttr[] = { 't', 'y', 'p', 'e', '\0' };
+		nsEmbedString typeVal;
+		linkElement->GetAttribute (nsEmbedString (typeAttr), typeVal);
+
+		nsEmbedCString cTypeVal;
+		NS_UTF16ToCString (typeVal, NS_CSTRING_ENCODING_UTF8, cTypeVal);
+
+		PRInt16 decision = 0;
+		rv = policy->ShouldLoad (nsIContentPolicy::TYPE_IMAGE,
+					 favUri, docUri, eventTarget,
+					 cTypeVal, nsnull,
+					 &decision);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+		if (decision != nsIContentPolicy::ACCEPT) return NS_OK;
+#else
+		PRBool shouldLoad = PR_FALSE;
+		rv = policy->ShouldLoad (nsIContentPolicy::IMAGE,
+					 favUri, eventTarget,
+					 domWin,
+					 &shouldLoad);
+		NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+		if (!shouldLoad) return NS_OK;
+#endif
+
+		/* ok, we accept this as a valid favicon for this site */
+		g_signal_emit_by_name (mOwner->mEmbed, "ge_favicon", faviconUrl.get());
 	}
 
-	return NS_OK;
-}	
-
-NS_IMETHODIMP
-EphyFaviconEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
-{
-	nsCOMPtr<nsIDOMEventTarget> eventTarget;
-	aDOMEvent->GetTarget(getter_AddRefs(eventTarget));
-
-	nsCOMPtr<nsIDOMNode> node = do_QueryInterface(eventTarget);
-	NS_ENSURE_TRUE (node, NS_ERROR_FAILURE);
-
-	HandleFaviconLink (node);
-
 	return NS_OK;
 }
 
 NS_IMETHODIMP
 EphyPopupBlockEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 {
-	NS_ENSURE_TRUE (mOwner != NULL, NS_ERROR_FAILURE);
-
 	nsCOMPtr<nsIDOMPopupBlockedEvent> popupEvent =
 		do_QueryInterface (aDOMEvent);
 	NS_ENSURE_TRUE (popupEvent, NS_ERROR_FAILURE);
@@ -206,7 +287,7 @@ EphyPopupBlockEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 			   NS_CSTRING_ENCODING_UTF8,
 			   popupWindowFeaturesString);
 
-	g_signal_emit_by_name(mOwner, "ge_popup_blocked",
+	g_signal_emit_by_name(mOwner->mEmbed, "ge_popup_blocked",
 			      popupWindowURIString.get(),
 			      popupWindowFeaturesString.get());
 
@@ -239,7 +320,7 @@ EphyModalAlertEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 	if (strcmp (cType.get(), "DOMWillOpenModalDialog") == 0)
 	{
 		gboolean retval = FALSE;
-		g_signal_emit_by_name (mOwner, "ge-modal-alert", &retval);
+		g_signal_emit_by_name (mOwner->mEmbed, "ge-modal-alert", &retval);
 
 		/* suppress alert */
 		if (retval)
@@ -250,31 +331,127 @@ EphyModalAlertEventListener::HandleEvent (nsIDOMEvent * aDOMEvent)
 	}
 	else if (strcmp (cType.get(), "DOMModalDialogClosed") == 0)
 	{
-		g_signal_emit_by_name (mOwner, "ge-modal-alert-closed");
+		g_signal_emit_by_name (mOwner->mEmbed, "ge-modal-alert-closed");
 	}
 #endif
 
 	return NS_OK;
 }
 
+EphyContextMenuListener::EphyContextMenuListener()
+: mOwner(nsnull)
+{
+	LOG ("EphyContextMenuListener ctor (%p)", this);
+}
+
+EphyContextMenuListener::~EphyContextMenuListener()
+{
+	LOG ("EphyContextMenuListener dtor (%p)", this);
+}
+
+NS_IMPL_ISUPPORTS1(EphyContextMenuListener, nsIDOMContextMenuListener)
+
+nsresult
+EphyContextMenuListener::Init(EphyBrowser *aOwner)
+{
+	mOwner = aOwner;
+	return NS_OK;
+}
+
+NS_IMETHODIMP
+EphyContextMenuListener::ContextMenu (nsIDOMEvent* aDOMEvent)
+{
+	nsCOMPtr<nsIDOMMouseEvent> mouseEvent = do_QueryInterface(aDOMEvent);
+	NS_ENSURE_TRUE (mouseEvent, NS_ERROR_FAILURE);
+
+	MozillaEmbedEvent *info;
+	info = mozilla_embed_event_new (NS_STATIC_CAST (gpointer, aDOMEvent));
+
+	nsresult rv;
+	EventContext context;
+	context.Init (mOwner);
+        rv = context.GetMouseEventInfo (mouseEvent, MOZILLA_EMBED_EVENT (info));
+
+	/* Don't do any magic handling if we can't actually show the context
+	 * menu, this can happen for XUL pages (e.g. about:config)
+	 */
+	if (NS_FAILED (rv))
+	{
+		g_object_unref (info);
+		return NS_OK;   
+	}
+
+	if (info->button == 0)
+	{
+		/* Translate relative coordinates to absolute values, and try
+		 * to avoid covering links by adding a little offset
+		 */
+		int x, y;
+		gdk_window_get_origin (GTK_WIDGET (mOwner->mEmbed)->window, &x, &y);
+		info->x += x + 6;       
+		info->y += y + 6;
+
+		// Set the keycode to something sensible
+		info->keycode = nsIDOMKeyEvent::DOM_VK_CONTEXT_MENU;
+	}
+
+	if (info->modifier == GDK_CONTROL_MASK)
+	{
+		info->context = EPHY_EMBED_CONTEXT_DOCUMENT;
+	}
+
+	gboolean retval = FALSE;
+	nsCOMPtr<nsIDOMDocument> domDoc;
+	rv = context.GetTargetDocument (getter_AddRefs(domDoc));
+	if (NS_SUCCEEDED(rv))
+	{
+		mOwner->PushTargetDocument (domDoc);
+
+		g_signal_emit_by_name (mOwner->mEmbed, "ge_context_menu",
+				       info, &retval);
+
+		mOwner->PopTargetDocument ();
+	}
+
+	/* We handled the event, block javascript calls */
+	if (retval)
+	{
+		aDOMEvent->PreventDefault();
+		aDOMEvent->StopPropagation();
+	}
+	
+	g_object_unref (info);
+
+	return NS_OK;
+}
+
+NS_IMETHODIMP
+EphyContextMenuListener::HandleEvent (nsIDOMEvent* aDOMEvent)
+{
+	return NS_ERROR_NOT_IMPLEMENTED;
+}
+
 EphyBrowser::EphyBrowser ()
 : mFaviconEventListener(nsnull)
 , mPopupBlockEventListener(nsnull)
 , mModalAlertListener(nsnull)
+, mContextMenuListener(nsnull)
 , mInitialized(PR_FALSE)
 {
-	LOG ("EphyBrowser ctor (%p)", this)
+	LOG ("EphyBrowser ctor (%p)", this);
 }
 
 EphyBrowser::~EphyBrowser ()
 {
-	LOG ("EphyBrowser dtor (%p)", this)
+	LOG ("EphyBrowser dtor (%p)", this);
 }
 
 nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 {
 	if (mInitialized) return NS_OK;
 
+	mEmbed = GTK_WIDGET (mozembed);
+
 	gtk_moz_embed_get_nsIWebBrowser (mozembed,
 					 getter_AddRefs(mWebBrowser));
 	NS_ENSURE_TRUE (mWebBrowser, NS_ERROR_FAILURE);
@@ -291,27 +468,79 @@ nsresult EphyBrowser::Init (GtkMozEmbed *mozembed)
 	mFaviconEventListener = new EphyFaviconEventListener();
 	if (!mFaviconEventListener) return NS_ERROR_OUT_OF_MEMORY;
 
-	rv = mFaviconEventListener->Init (EPHY_EMBED (mozembed));
+	rv = mFaviconEventListener->Init (this);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	mPopupBlockEventListener = new EphyPopupBlockEventListener();
 	if (!mPopupBlockEventListener) return NS_ERROR_OUT_OF_MEMORY;
 
-	rv = mPopupBlockEventListener->Init (EPHY_EMBED (mozembed));
+	rv = mPopupBlockEventListener->Init (this);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	mModalAlertListener = new EphyModalAlertEventListener ();
 	if (!mModalAlertListener) return NS_ERROR_OUT_OF_MEMORY;
 
-	rv = mModalAlertListener->Init (EPHY_EMBED (mozembed));
+	rv = mModalAlertListener->Init (this);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	mContextMenuListener = new EphyContextMenuListener();
+	if (!mContextMenuListener) return NS_ERROR_OUT_OF_MEMORY;
+
+	rv = mContextMenuListener->Init (this);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
  	rv = GetListener();
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
+	rv = AttachListeners();
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+#ifdef HAVE_MOZILLA_PSM
+	/* FIXME: mozilla sucks! nsWebBrowser already has an instance of this,
+	 * but we cannot get to it!
+	 * See https://bugzilla.mozilla.org/show_bug.cgi?id=94974
+	 */
+	/* First try GI */
+	mSecurityInfo = do_GetInterface (mWebBrowser);
+	/* Try to instantiate it under the re-registered contract ID */
+	if (!mSecurityInfo)
+	{
+		/* This will cause all security warning dialogs to be shown
+		 * twice (once by this instance, and another time by nsWebBrowser's
+		 * instance of nsSecurityBrowserUIImpl), but there appears to be
+		 * no other way :-(
+		 */
+		mSecurityInfo = do_CreateInstance("@gnome.org/project/epiphany/hacks/secure-browser-ui;1", &rv);
+		if (NS_SUCCEEDED (rv) && mSecurityInfo)
+		{
+			rv = mSecurityInfo->Init (mDOMWindow);
+			NS_ENSURE_SUCCESS (rv, rv);
+		}
+	}
+	/* Try the original contract ID */
+	if (!mSecurityInfo)
+	{
+		/* This will cause all security warning dialogs to be shown
+		 * twice (once by this instance, and another time by nsWebBrowser's
+		 * instance of nsSecurityBrowserUIImpl), but there appears to be
+		 * no other way :-(
+		 */
+		mSecurityInfo = do_CreateInstance(NS_SECURE_BROWSER_UI_CONTRACTID, &rv);
+		if (NS_SUCCEEDED (rv) && mSecurityInfo)
+		{
+			rv = mSecurityInfo->Init (mDOMWindow);
+			NS_ENSURE_SUCCESS (rv, rv);
+		}
+	}
+	if (!mSecurityInfo)
+	{
+		g_warning ("Failed to instantiate nsISecureBrowserUI!\n");
+ 	}
+#endif
+
 	mInitialized = PR_TRUE;
 
-	return AttachListeners();
+	return NS_OK;
 }
 
 nsresult
@@ -345,6 +574,8 @@ EphyBrowser::AttachListeners(void)
 					     mModalAlertListener, PR_TRUE);
 	rv |= mEventTarget->AddEventListener(nsEmbedString(DOMModalDialogClosed),
 					     mModalAlertListener, PR_TRUE);
+	rv |= mEventTarget->AddEventListener(nsEmbedString(ContextMenu),
+					     mContextMenuListener, PR_TRUE /* capture */);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	return NS_OK;
@@ -364,6 +595,8 @@ EphyBrowser::DetachListeners(void)
 						mModalAlertListener, PR_TRUE);
 	rv |= mEventTarget->RemoveEventListener(nsEmbedString(DOMModalDialogClosed),
 						mModalAlertListener, PR_TRUE);
+	rv |= mEventTarget->RemoveEventListener(nsEmbedString(ContextMenu),
+					        mContextMenuListener, PR_TRUE /* capture */);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
 	return NS_OK;
@@ -461,6 +694,7 @@ nsresult EphyBrowser::Destroy ()
       	mWebBrowser = nsnull;
 	mDOMWindow = nsnull;
 	mEventTarget = nsnull;
+	mEmbed = nsnull;
 
 	mInitialized = PR_FALSE;
 
@@ -996,16 +1230,21 @@ nsresult EphyBrowser::GetHasModifiedForms (PRBool *modified)
 }
 
 nsresult
-EphyBrowser::SetSecurityInfo (nsIRequest *aRequest)
+EphyBrowser::GetSecurityInfo (PRUint32 *aState, nsACString &aDescription)
 {
 #ifdef HAVE_MOZILLA_PSM
-	/* clear previous security info */
-	mSecurityInfo = nsnull;
+	NS_ENSURE_TRUE (mSecurityInfo, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsIChannel> channel (do_QueryInterface (aRequest));
-	NS_ENSURE_TRUE (channel, NS_ERROR_FAILURE);
+	nsresult rv;
+	rv = mSecurityInfo->GetState (aState);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
 
-	channel->GetSecurityInfo (getter_AddRefs (mSecurityInfo));
+	nsEmbedString tooltip;
+	rv = mSecurityInfo->GetTooltipText (tooltip);
+	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+	NS_UTF16ToCString (tooltip,
+			   NS_CSTRING_ENCODING_UTF8, aDescription);
 
 	return NS_OK;
 #else
@@ -1014,25 +1253,61 @@ EphyBrowser::SetSecurityInfo (nsIRequest *aRequest)
 }
 
 nsresult
-EphyBrowser::GetSecurityDescription (nsACString &aDescription)
+EphyBrowser::ShowCertificate ()
 {
 #ifdef HAVE_MOZILLA_PSM
-	if (!mSecurityInfo) return NS_ERROR_FAILURE;
+	NS_ENSURE_TRUE (mSecurityInfo, NS_ERROR_FAILURE);
 
-	nsCOMPtr<nsITransportSecurityInfo> tsInfo (do_QueryInterface (mSecurityInfo));
-	NS_ENSURE_TRUE (tsInfo, NS_ERROR_FAILURE);
+	nsCOMPtr<nsISSLStatusProvider> statusProvider (do_QueryInterface (mSecurityInfo));
+	NS_ENSURE_TRUE (statusProvider, NS_ERROR_FAILURE);
 
-	nsresult rv;
-	PRUnichar *tooltip = nsnull;
-	rv = tsInfo->GetShortSecurityDescription (&tooltip);
-	NS_ENSURE_TRUE (NS_SUCCEEDED (rv) && tooltip, NS_ERROR_FAILURE);
+	nsCOMPtr<nsISSLStatus> SSLStatus;
+	statusProvider->GetSSLStatus (getter_AddRefs (SSLStatus));
+	NS_ENSURE_TRUE (SSLStatus, NS_ERROR_FAILURE);
 
-	NS_UTF16ToCString (nsEmbedString (tooltip),
-			   NS_CSTRING_ENCODING_UTF8, aDescription);
-	if (tooltip) nsMemory::Free (tooltip);
+	nsCOMPtr<nsIX509Cert> serverCert;
+	SSLStatus->GetServerCert (getter_AddRefs (serverCert));
+	NS_ENSURE_TRUE (serverCert, NS_ERROR_FAILURE);
 
-	return NS_OK;
+	nsCOMPtr<nsICertificateDialogs> certDialogs (do_GetService (NS_CERTIFICATEDIALOGS_CONTRACTID));
+	NS_ENSURE_TRUE (certDialogs, NS_ERROR_FAILURE);
+
+	nsCOMPtr<nsIInterfaceRequestor> requestor(do_QueryInterface (mDOMWindow));
+
+	return certDialogs->ViewCert (requestor, serverCert);
 #else
-	return NS_ERROR_NOT_IMPLEMENTED;
+	return NS_OK;
 #endif
 }
+
+EphyEmbedDocumentType
+EphyBrowser::GetDocumentType ()
+{
+  EphyEmbedDocumentType type = EPHY_EMBED_DOCUMENT_OTHER;
+
+  NS_ENSURE_TRUE (mDOMWindow, type);
+
+  nsresult rv;
+  nsCOMPtr<nsIDOMDocument> domDoc;
+  rv = GetDocument (getter_AddRefs (domDoc));
+  NS_ENSURE_SUCCESS (rv, type);
+
+  nsCOMPtr<nsIDOMHTMLDocument> htmlDoc (do_QueryInterface (domDoc));
+  nsCOMPtr<nsIDOMXMLDocument> xmlDoc (do_QueryInterface (domDoc));
+  nsCOMPtr<nsIImageDocument> imgDoc (do_QueryInterface (domDoc));
+
+  if (xmlDoc)
+    {
+      type = EPHY_EMBED_DOCUMENT_XML;
+    }
+  else if (imgDoc)
+    {
+      type = EPHY_EMBED_DOCUMENT_IMAGE;
+    }
+  else if (htmlDoc)
+    {
+      type = EPHY_EMBED_DOCUMENT_HTML;
+    }
+
+  return type;
+}
