@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EventContext.cpp,v 1.48 2005/01/23 15:38:02 chpe Exp $
+ *  $Id: EventContext.cpp,v 1.50 2005/02/27 14:13:45 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -403,15 +403,18 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 				{
 					/* cut "mailto:" */
 					href.Cut (0, 7);
-					// FIXME: cut any chars after "?"
+
+					char *str = g_strdup (href.get());
+					g_strdelimit (str, "?", '\0');
 
 					nsEmbedCString unescapedHref;
-					rv = Unescape (href, unescapedHref);
+					rv = Unescape (nsEmbedCString(str), unescapedHref);
 					if (NS_SUCCEEDED (rv) && unescapedHref.Length())
 					{
 						SetStringProperty ("email", unescapedHref.get());
 						info->context |= EPHY_EMBED_CONTEXT_EMAIL_LINK;
 					}
+					g_free (str);
 				}
 				
 				if (anchor && tmp.Length()) 
@@ -810,6 +813,11 @@ nsresult EventContext::CheckInput (nsIDOMNode *aNode)
 		 g_ascii_strcasecmp (value.get(), "checkbox") != 0)
 	{
 		mEmbedEvent->context |= EPHY_EMBED_CONTEXT_INPUT;
+
+		if (g_ascii_strcasecmp (value.get(), "password") == 0)
+		{
+			mEmbedEvent->context |= EPHY_EMBED_CONTEXT_INPUT_PASSWORD;
+		}
 	}
 
 	return NS_OK;
