@@ -38,7 +38,7 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.64 2005/02/27 18:50:45 chpe Exp $
+ * $Id: MozDownload.cpp,v 1.65.2.1 2005/03/23 13:25:16 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -403,11 +403,11 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 
 			/* HACK we use the application description to decide
 			   if we have to open the saved file */
-			if (g_str_has_suffix (cDesc.get(), "gnome-default:"))
+			if (g_str_has_prefix (cDesc.get(), "gnome-default:"))
 			{
 				/* Format gnome-default:<usertime>:<helperapp id> */
 				char **str = g_strsplit (cDesc.get(), ":", -1);
-				g_return_val_if_fail (g_strv_length (str) != 3, NS_ERROR_FAILURE);
+				g_return_val_if_fail (g_strv_length (str) == 3, NS_ERROR_FAILURE);
 
 				char *end;
 				guint32 user_time = strtoul (str[1], &end, 0);
@@ -763,6 +763,8 @@ nsresult BuildDownloadPath (const char *defaultFileName, nsILocalFile **_retval)
 	}
 
 	nsCOMPtr <nsILocalFile> destFile (do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
+	NS_ENSURE_TRUE (destFile, NS_ERROR_FAILURE);
+
 	destFile->InitWithNativePath (nsEmbedCString (path));
 	g_free (path);
 
