@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.53 2004/11/07 19:32:17 chpe Exp $
+ *  $Id: ephy-node-view.c,v 1.58 2005/01/21 19:33:02 jbaayen Exp $
  */
 
 #include "config.h"
@@ -36,8 +36,7 @@
 #include "ephy-dnd.h"
 #include "ephy-gui.h"
 #include "ephy-marshal.h"
-#include "ephy-string.h"
-#include "string.h"
+#include <string.h>
 
 static void ephy_node_view_class_init (EphyNodeViewClass *klass);
 static void ephy_node_view_init (EphyNodeView *view);
@@ -45,7 +44,7 @@ static void ephy_node_view_finalize (GObject *object);
 
 #define EPHY_NODE_VIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_NODE_VIEW, EphyNodeViewPrivate))
 
-struct EphyNodeViewPrivate
+struct _EphyNodeViewPrivate
 {
 	EphyNode *root;
 
@@ -739,7 +738,7 @@ motion_notify_cb (GtkWidget *widget,
 		{
 			context = gtk_drag_begin
 				(widget, view->priv->source_target_list,
-                                 GDK_ACTION_COPY,
+                                 GDK_ACTION_ASK | GDK_ACTION_COPY | GDK_ACTION_LINK,
                                  view->priv->drag_button,
                                  (GdkEvent*)event);
 
@@ -1159,13 +1158,14 @@ ephy_node_view_add_data_column (EphyNodeView *view,
 	return column;
 }
 
-GtkTreeViewColumn *
+int
 ephy_node_view_add_column (EphyNodeView *view,
 			   const char  *title,
 			   GType value_type,
 			   guint prop_id,
 			   EphyNodeViewFlags flags,
-			   EphyTreeModelNodeValueFunc icon_func)
+			   EphyTreeModelNodeValueFunc icon_func,
+			   GtkTreeViewColumn **ret)
 
 {
 	GtkTreeViewColumn *gcolumn;
@@ -1237,7 +1237,10 @@ ephy_node_view_add_column (EphyNodeView *view,
 		gtk_tree_view_column_set_sort_column_id (gcolumn, column);
 	}
 
-	return gcolumn;
+	if (ret != NULL)
+		*ret = gcolumn;
+
+	return column;
 }
 
 void
@@ -1410,7 +1413,8 @@ void
 ephy_node_view_enable_drag_source (EphyNodeView *view,
 				   GtkTargetEntry *types,
 				   int n_types,
-				   int column)
+				   int base_drag_column,
+				   int extra_drag_column)
 {
 	GtkWidget *treeview;
 
@@ -1421,8 +1425,10 @@ ephy_node_view_enable_drag_source (EphyNodeView *view,
 	view->priv->source_target_list =
 		gtk_target_list_new (types, n_types);
 
-	ephy_tree_model_sort_set_column_id (EPHY_TREE_MODEL_SORT (view->priv->sortmodel),
-					    column);
+	ephy_tree_model_sort_set_base_drag_column_id  (EPHY_TREE_MODEL_SORT (view->priv->sortmodel),
+					               base_drag_column);
+	ephy_tree_model_sort_set_extra_drag_column_id (EPHY_TREE_MODEL_SORT (view->priv->sortmodel),
+					               extra_drag_column);
 
 	g_signal_connect_object (G_OBJECT (view),
 			         "button_release_event",
@@ -1479,23 +1485,6 @@ ephy_node_view_is_target (EphyNodeView *view)
 	return gtk_widget_is_focus (GTK_WIDGET (view));
 }
 
-gboolean
-ephy_node_view_has_selection (EphyNodeView *view, gboolean *multiple)
-{
-	GtkTreeSelection *selection;
-	int rows;
-
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
-	rows = gtk_tree_selection_count_selected_rows (selection);
-
-	if (multiple)
-	{
-		*multiple = rows > 1;
-	}
-
-	return rows > 0;
-}
-
 static gboolean
 filter_visible_func (GtkTreeModel *model, GtkTreeIter *iter, gpointer data)
 {
