@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-new-bookmark.c,v 1.52 2004/12/19 15:15:47 chpe Exp $
+ *  $Id: ephy-new-bookmark.c,v 1.54 2005/01/28 22:25:02 chpe Exp $
  */
 
 #include "config.h"
@@ -63,7 +63,7 @@ enum
 	RESPONSE_NEW_TOPIC
 };
 
-struct EphyNewBookmarkPrivate
+struct _EphyNewBookmarkPrivate
 {
 	EphyBookmarks *bookmarks;
 	char *location;
@@ -466,7 +466,7 @@ ephy_new_bookmark_set_title (EphyNewBookmark *bookmark,
 {
 	const char *real_title;
 
-	LOG ("Setting new bookmark title to: \"%s\"", title)
+	LOG ("Setting new bookmark title to: \"%s\"", title);
 
 	if (title == NULL || strlen (title) == 0)
 	{
